@@ -3,40 +3,39 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
-__author__ = 'Satrajit S. Ghosh'
-__email__ = 'satra@mit.edu'
-
-import logging
-logger = logging.getLogger('rdf')
-
-from collections import defaultdict, OrderedDict
+import base64
+from collections import OrderedDict
 import datetime
 import io
 
-from prov.serializers import Serializer, Error
-from prov.constants import *
-from prov.model import (Literal, Identifier, QualifiedName,
-                        Namespace, ProvDocument, ProvBundle, first,
-                        parse_xsd_datetime, ProvRecord)
-
-from six import text_type
-
-import base64
-import datetime
 import dateutil.parser
-import prov.model as pm
-
-attr2rdf = lambda attr: URIRef(PROV[PROV_ID_ATTRIBUTES_MAP[attr].split('prov:')[1]].uri)
+import six
 
 from rdflib.term import URIRef, BNode
 from rdflib.term import Literal as RDFLiteral
 from rdflib.graph import ConjunctiveGraph
 from rdflib.namespace import RDF, RDFS, XSD
 
+import prov.model as pm
+from prov.constants import (
+    PROV, PROV_ID_ATTRIBUTES_MAP, PROV_N_MAP, PROV_BASE_CLS, XSD_QNAME,
+    PROV_END, PROV_START, PROV_USAGE, PROV_GENERATION, PROV_DERIVATION, PROV_INVALIDATION,
+    PROV_ALTERNATE, PROV_MENTION, PROV_DELEGATION, PROV_ACTIVITY, PROV_ATTR_STARTTIME,
+    PROV_ATTR_ENDTIME, PROV_LOCATION, PROV_ATTR_TIME, PROV_ROLE, PROV_COMMUNICATION,
+    PROV_ATTR_INFORMANT, PROV_ATTR_RESPONSIBLE, PROV_ATTR_TRIGGER, PROV_ATTR_ENDER,
+    PROV_ATTR_STARTER, PROV_ATTR_USED_ENTITY, PROV_ASSOCIATION)
+from prov.serializers import Serializer, Error
+
+
+__author__ = 'Satrajit S. Ghosh'
+__email__ = 'satra@mit.edu'
+
+
 class ProvRDFException(Error):
     pass
 
-class AnonymousIDGenerator():
+
+class AnonymousIDGenerator:
     def __init__(self):
         self._cache = {}
         self._count = 0
@@ -44,8 +43,9 @@ class AnonymousIDGenerator():
     def get_anon_id(self, obj, local_prefix="id"):
         if obj not in self._cache:
             self._count += 1
-            self._cache[obj] = Identifier('_:%s%d' % (local_prefix,
-                                                      self._count)).uri
+            self._cache[obj] = pm.Identifier(
+                '_:%s%d' % (local_prefix, self._count)
+            ).uri
         return self._cache[obj]
 
 
@@ -53,7 +53,7 @@ class AnonymousIDGenerator():
 LITERAL_XSDTYPE_MAP = {
     float: XSD['double'],
     int: XSD['int'],
-    text_type: XSD['string'],
+    six.text_type: XSD['string'],
     # boolean, string values are supported natively by PROV-RDF
     # datetime values are converted separately
 }
@@ -63,6 +63,10 @@ if six.integer_types[-1] not in LITERAL_XSDTYPE_MAP:
     LITERAL_XSDTYPE_MAP[six.integer_types[-1]] = XSD['long']
 
 
+def attr2rdf(attr):
+    return URIRef(PROV[PROV_ID_ATTRIBUTES_MAP[attr].split('prov:')[1]].uri)
+
+
 def valid_qualified_name(bundle, value, xsd_qname=False):
     if value is None:
         return None
@@ -78,23 +82,15 @@ class ProvRDFSerializer(Serializer):
     def serialize(self, stream=None, rdf_format='trig', **kwargs):
         """
         Serializes a :class:`~prov.model.ProvDocument` instance to
-        `Prov-O <https://www.w3.org/TR/prov-o/>`_.
+        `PROV-O <https://www.w3.org/TR/prov-o/>`_.
 
         :param stream: Where to save the output.
+        :param rdf_format: The RDF format of the output, default to TRiG.
         """
         container = self.encode_document(self.document)
         newargs = kwargs.copy()
         newargs['format'] = rdf_format
 
-        if newargs['format'] == 'trig':
-            gr = ConjunctiveGraph()
-            gr.context_aware = True
-            gr.parse(data=container.serialize(format='nquads'), format='nquads')
-            for namespace in container.namespaces():
-                if namespace not in list(gr.namespaces()):
-                    gr.bind(namespace[0], namespace[1])
-            container = gr
-
         if six.PY2:
             buf = io.BytesIO()
             try:
@@ -120,22 +116,23 @@ class ProvRDFSerializer(Serializer):
                 if isinstance(stream, io.TextIOBase):
                     stream.write(buf.read().decode('utf-8'))
                 else:
-                    stream.write(buf.read()) #.encode('utf-8'))
+                    stream.write(buf.read())
             finally:
                 buf.close()
 
     def deserialize(self, stream, rdf_format='trig', **kwargs):
         """
-        Deserialize from the `Prov-O <https://www.w3.org/TR/prov-o/>`_
+        Deserialize from the `PROV-O <https://www.w3.org/TR/prov-o/>`_
         representation to a :class:`~prov.model.ProvDocument` instance.
 
         :param stream: Input data.
+        :param rdf_format: The RDF format of the input data, default: TRiG.
         """
         newargs = kwargs.copy()
         newargs['format'] = rdf_format
         container = ConjunctiveGraph()
         container.parse(stream, **newargs)
-        document = ProvDocument()
+        document = pm.ProvDocument()
         self.document = document
         self.decode_document(container, document)
         return document
@@ -146,20 +143,19 @@ class ProvRDFSerializer(Serializer):
     def encode_rdf_representation(self, value):
         if isinstance(value, URIRef):
             return value
-        elif isinstance(value, Literal):
+        elif isinstance(value, pm.Literal):
             return literal_rdf_representation(value)
         elif isinstance(value, datetime.datetime):
             return RDFLiteral(value.isoformat(), datatype=XSD['dateTime'])
-        elif isinstance(value, QualifiedName):
+        elif isinstance(value, pm.QualifiedName):
             return URIRef(value.uri)
-        elif isinstance(value, Identifier):
+        elif isinstance(value, pm.Identifier):
             return RDFLiteral(value.uri, datatype=XSD['anyURI'])
         elif type(value) in LITERAL_XSDTYPE_MAP:
             return RDFLiteral(value, datatype=LITERAL_XSDTYPE_MAP[type(value)])
         else:
             return RDFLiteral(value)
 
-
     def decode_rdf_representation(self, literal, graph):
         if isinstance(literal, RDFLiteral):
             value = literal.value if literal.value is not None else literal
@@ -173,10 +169,18 @@ class ProvRDFSerializer(Serializer):
                 return pm.Literal(literal, datatype=XSD_QNAME)
             if datatype == XSD['dateTime']:
                 return dateutil.parser.parse(literal)
+            if datatype == XSD['gYear']:
+                return pm.Literal(dateutil.parser.parse(literal).year,
+                                  datatype=self.valid_identifier(datatype))
+            if datatype == XSD['gYearMonth']:
+                parsed_info = dateutil.parser.parse(literal)
+                return pm.Literal('{0}-{1:02d}'.format(parsed_info.year, parsed_info.month),
+                                  datatype=self.valid_identifier(datatype))
             else:
                 # The literal of standard Python types is not converted here
-                # It will be automatically converted when added to a record by _auto_literal_conversion()
-                return Literal(value, self.valid_identifier(datatype), langtag)
+                # It will be automatically converted when added to a record by
+                # _auto_literal_conversion()
+                return pm.Literal(value, self.valid_identifier(datatype), langtag)
         elif isinstance(literal, URIRef):
             rval = self.valid_identifier(literal)
             if rval is None:
@@ -212,7 +216,7 @@ class ProvRDFSerializer(Serializer):
         for record in bundle._records:
             rec_type = record.get_type()
             if hasattr(record, 'identifier') and record.identifier:
-                identifier = URIRef(text_type(real_or_anon_id(record)))
+                identifier = URIRef(six.text_type(real_or_anon_id(record)))
                 container.add((identifier, RDF.type, URIRef(rec_type.uri)))
             else:
                 identifier = None
@@ -245,15 +249,18 @@ class ProvRDFSerializer(Serializer):
                                 try:
                                     obj_val = record.formal_attributes[1][1]
                                     obj_attr = URIRef(record.formal_attributes[1][0].uri)
+                                    # TODO: Why is obj_attr above not used anywhere?
                                 except IndexError:
                                     obj_val = None
-                                if obj_val and (rec_type not in [PROV_END,
-                                                                PROV_START,
-                                                                PROV_USAGE,
-                                                                PROV_GENERATION,
-                                                                PROV_DERIVATION,
-                                                                PROV_INVALIDATION] or
-                                                (valid_formal_indices == {0, 1} and
+                                if obj_val and (rec_type not in
+                                                    {PROV_END,
+                                                     PROV_START,
+                                                     PROV_USAGE,
+                                                     PROV_GENERATION,
+                                                     PROV_DERIVATION,
+                                                     PROV_ASSOCIATION,
+                                                     PROV_INVALIDATION} or
+                                            (valid_formal_indices == {0, 1} and
                                                  len(record.extra_attributes) == 0)):
                                     used_objects.append(record.formal_attributes[1][0])
                                     obj_val = self.encode_rdf_representation(obj_val)
@@ -266,9 +273,9 @@ class ProvRDFSerializer(Serializer):
                                             obj_val = self.encode_rdf_representation(record.formal_attributes[2][1])
                                             container.add((subj, URIRef(PROV['asInBundle'].uri), obj_val))
                                         has_qualifiers = False
-                            if rec_type in [PROV_ALTERNATE]: #, PROV_ASSOCIATION]:
+                            if rec_type in [PROV_ALTERNATE]:
                                 continue
-                            if subj and (has_qualifiers or identifier):  #and (len(record.extra_attributes) > 0 or                                                            identifier):
+                            if subj and (has_qualifiers or identifier):
                                 qualifier = rec_type._localpart
                                 rec_uri = rec_type.uri
                                 for attr_name, val in record.extra_attributes:
@@ -288,9 +295,9 @@ class ProvRDFSerializer(Serializer):
                                 else:
                                     bnode = identifier = BNode()
                                     container.add((subj, QRole, identifier))
-                                    container.add((identifier, RDF.type,
-                                                   URIRef(rec_uri)))
-                                               # reset identifier to BNode
+                                    container.add(
+                                        (identifier, RDF.type, URIRef(rec_uri))
+                                    )  # reset identifier to BNode
                         if value is not None and attr not in used_objects:
                             if attr in formal_objects:
                                 pred = attr2rdf(attr)
@@ -302,7 +309,7 @@ class ProvRDFSerializer(Serializer):
                                 pred = RDF.type
                             elif attr == PROV['label']:
                                 pred = RDFS.label
-                            elif isinstance(attr, QualifiedName):
+                            elif isinstance(attr, pm.QualifiedName):
                                 pred = URIRef(attr.uri)
                             else:
                                 pred = self.encode_rdf_representation(attr)
@@ -312,10 +319,13 @@ class ProvRDFSerializer(Serializer):
                                 pred = URIRef(PROV['activity'].uri)
                             if PROV['responsible'].uri in pred:
                                 pred = URIRef(PROV['agent'].uri)
-                            if rec_type == PROV_DELEGATION and PROV['activity'].uri in pred:
+                            if rec_type == PROV_DELEGATION and \
+                                            PROV['activity'].uri in pred:
                                 pred = URIRef(PROV['hadActivity'].uri)
-                            if (rec_type in [PROV_END, PROV_START] and PROV['trigger'].uri in pred) or\
-                                (rec_type in [PROV_USAGE] and PROV['used'].uri in pred):
+                            if (rec_type in [PROV_END, PROV_START] and
+                                            PROV['trigger'].uri in pred) or\
+                                (rec_type in [PROV_USAGE] and
+                                         PROV['used'].uri in pred):
                                 pred = URIRef(PROV['entity'].uri)
                             if rec_type in [PROV_GENERATION, PROV_END,
                                             PROV_START, PROV_USAGE,
@@ -347,15 +357,15 @@ class ProvRDFSerializer(Serializer):
                         continue
                     if value is None:
                         continue
-                    if isinstance(value, ProvRecord):
-                        obj = URIRef(text_type(real_or_anon_id(value)))
+                    if isinstance(value, pm.ProvRecord):
+                        obj = URIRef(six.text_type(real_or_anon_id(value)))
                     else:
                         #  Assuming this is a datetime value
                         obj = self.encode_rdf_representation(value)
                     if attr == PROV['location']:
                         pred = URIRef(PROV['atLocation'].uri)
-                        if False and isinstance(value, (URIRef, QualifiedName)):
-                            if isinstance(value, QualifiedName):
+                        if False and isinstance(value, (URIRef, pm.QualifiedName)):
+                            if isinstance(value, pm.QualifiedName):
                                 value = URIRef(value.uri)
                             container.add((identifier, pred, value))
                         else:
@@ -377,13 +387,13 @@ class ProvRDFSerializer(Serializer):
 
     def decode_document(self, content, document):
         for prefix, url in content.namespaces():
-            document.add_namespace(prefix, text_type(url))
+            document.add_namespace(prefix, six.text_type(url))
         if hasattr(content, 'contexts'):
             for graph in content.contexts():
                 if isinstance(graph.identifier, BNode):
                     self.decode_container(graph, document)
                 else:
-                    bundle_id = text_type(graph.identifier)
+                    bundle_id = six.text_type(graph.identifier)
                     bundle = document.bundle(bundle_id)
                     self.decode_container(graph, bundle)
         else:
@@ -425,8 +435,8 @@ class ProvRDFSerializer(Serializer):
                             }
         other_attributes = {}
         for stmt in graph.triples((None, RDF.type, None)):
-            id = text_type(stmt[0])
-            obj = text_type(stmt[2])
+            id = six.text_type(stmt[0])
+            obj = six.text_type(stmt[2])
             if obj in PROV_CLS_MAP:
                 if not isinstance(stmt[0], BNode) and self.valid_identifier(id) is None:
                     prefix, iri, _ = graph.namespace_manager.compute_qname(id)
@@ -458,7 +468,7 @@ class ProvRDFSerializer(Serializer):
                 obj = self.decode_rdf_representation(stmt[2], graph)
                 other_attributes[id].append((pm.PROV['type'], obj))
         for id, pred, obj in graph:
-            id = text_type(id)
+            id = six.text_type(id)
             if id not in other_attributes:
                 other_attributes[id] = []
             if pred == RDF.type:
@@ -470,20 +480,20 @@ class ProvRDFSerializer(Serializer):
                     mentionBundle = None
                     for stmt in graph.triples((URIRef(id), URIRef(pm.PROV['asInBundle'].uri), None)):
                         mentionBundle = stmt[2]
-                    getattr(bundle, relation_mapper[pred])(id, text_type(obj), mentionBundle)
+                    getattr(bundle, relation_mapper[pred])(id, six.text_type(obj), mentionBundle)
                 elif 'actedOnBehalfOf' in pred or 'wasAssociatedWith' in pred:
                     qualifier = 'qualified' + relation_mapper[pred].upper()[0] + relation_mapper[pred][1:]
                     qualifier_bnode = None
                     for stmt in graph.triples((URIRef(id), URIRef(pm.PROV[qualifier].uri), None)):
                         qualifier_bnode = stmt[2]
                     if qualifier_bnode is None:
-                        getattr(bundle, relation_mapper[pred])(id, text_type(obj))
+                        getattr(bundle, relation_mapper[pred])(id, six.text_type(obj))
                     else:
-                        fakeys = list(formal_attributes[text_type(qualifier_bnode)].keys())
-                        formal_attributes[text_type(qualifier_bnode)][fakeys[0]] = id
-                        formal_attributes[text_type(qualifier_bnode)][fakeys[1]] = text_type(obj)
+                        fakeys = list(formal_attributes[six.text_type(qualifier_bnode)].keys())
+                        formal_attributes[six.text_type(qualifier_bnode)][fakeys[0]] = id
+                        formal_attributes[six.text_type(qualifier_bnode)][fakeys[1]] = six.text_type(obj)
                 else:
-                    getattr(bundle, relation_mapper[pred])(id, text_type(obj))
+                    getattr(bundle, relation_mapper[pred])(id, six.text_type(obj))
             elif id in ids:
                 obj1 = self.decode_rdf_representation(obj, graph)
                 if obj is not None and obj1 is None:
@@ -491,29 +501,29 @@ class ProvRDFSerializer(Serializer):
                 pred_new = pred
                 if pred in predicate_mapper:
                     pred_new = predicate_mapper[pred]
-                if ids[id] == PROV_COMMUNICATION and 'activity' in text_type(pred_new):
+                if ids[id] == PROV_COMMUNICATION and 'activity' in six.text_type(pred_new):
                     pred_new = PROV_ATTR_INFORMANT
-                if ids[id] == PROV_DELEGATION and 'agent' in text_type(pred_new):
+                if ids[id] == PROV_DELEGATION and 'agent' in six.text_type(pred_new):
                     pred_new = PROV_ATTR_RESPONSIBLE
-                if ids[id] in [PROV_END, PROV_START] and 'entity' in text_type(pred_new):
+                if ids[id] in [PROV_END, PROV_START] and 'entity' in six.text_type(pred_new):
                     pred_new = PROV_ATTR_TRIGGER
-                if ids[id] in [PROV_END] and 'activity' in text_type(pred_new):
+                if ids[id] in [PROV_END] and 'activity' in six.text_type(pred_new):
                     pred_new = PROV_ATTR_ENDER
-                if ids[id] in [PROV_START] and 'activity' in text_type(pred_new):
+                if ids[id] in [PROV_START] and 'activity' in six.text_type(pred_new):
                     pred_new = PROV_ATTR_STARTER
-                if ids[id] == PROV_DERIVATION and 'entity' in text_type(pred_new):
+                if ids[id] == PROV_DERIVATION and 'entity' in six.text_type(pred_new):
                     pred_new = PROV_ATTR_USED_ENTITY
-                if text_type(pred_new) in [val.uri for val in formal_attributes[id]]:
+                if six.text_type(pred_new) in [val.uri for val in formal_attributes[id]]:
                     qname_key = self.valid_identifier(pred_new)
                     formal_attributes[id][qname_key] = obj1
                     unique_sets[id][qname_key].append(obj1)
                     if len(unique_sets[id][qname_key]) > 1:
                         formal_attributes[id][qname_key] = None
                 else:
-                    if 'qualified' not in text_type(pred_new) and \
-                                    'asInBundle' not in text_type(pred_new):
-                        other_attributes[id].append((text_type(pred_new), obj1))
-            local_key = text_type(obj)
+                    if 'qualified' not in six.text_type(pred_new) and \
+                                    'asInBundle' not in six.text_type(pred_new):
+                        other_attributes[id].append((six.text_type(pred_new), obj1))
+            local_key = six.text_type(obj)
             if local_key in ids:
                 if 'qualified' in pred:
                     formal_attributes[local_key][list(formal_attributes[local_key].keys())[0]] = id
@@ -543,9 +553,9 @@ class ProvRDFSerializer(Serializer):
 def walk(children, level=0, path=None, usename=True):
     """Generate all the full paths in a tree, as a dict.
 
-    Examples
-    --------
-    >>> from nipype.pipeline.engine.utils import walk
+    :Example:
+
+    >>> from prov.serializers.provrdf import walk
     >>> iterables = [('a', lambda: [1, 2]), ('b', lambda: [3, 4])]
     >>> [val['a'] for val in walk(iterables)]
     [1, 1, 2, 2]
@@ -574,7 +584,7 @@ def walk(children, level=0, path=None, usename=True):
 
 
 def literal_rdf_representation(literal):
-    value = text_type(literal.value) if literal.value else literal
+    value = six.text_type(literal.value) if literal.value else literal
     if literal.langtag:
         #  a language tag can only go with prov:InternationalizedString
         return RDFLiteral(value, lang=str(literal.langtag))
